import IHighlightOptions from './models/highlight-options';
import Config from './services/config';
export interface IHighlighted {
    element: HTMLElement;
    /**
     * Invokes the `UIGuide.clear` internally.
     */
    unhighlight: () => void;
}
export default class UIGuide {
    /**
     * Configure the global highlight settings.
     *
     * **Important**: *Updating the configuration while there's an on going*
     * *highlight operation is prohibited.*
     *
     * @param config New configuration.
     */
    static configure(configuration: Parameters<Config['update']>[number]): void;
    /**
     * Highlight an element from the page.
     *
     * **Tip:** *It is possible to highlight a new element without*
     * *`unhighlight`ing the current highlighted. Just make sure to wait your*
     * *first highlight operation before you can highlight a new one, otherwise*
     * *it will throw an error.*
     *
     * *Example:*
     * ```javascript
     * UIGuide.highlight('#target-1').then(() => {
     *   UIGuide.highlight('#target-2')
     * })
     * ```
     *
     * @param opts Target element or highlight options for advanced highlighting.
     */
    static highlight(opts: IHighlightOptions | Element | string): Promise<IHighlighted>;
    /**
     * Unhighlight the current highlighted element.
     *
     * You usually invoke this function after you're done highlighting all your
     * target elements.
     *
     * *Example:*
     * ```typescript
     * UIGuide.highlight('#first-target').then(() => {
     *   UIGuide.highlight('#final-target').then((highlighted) => {
     *     UIGuide.clear() // or highlighted.unhighlight()
     *   })
     * })
     * ```
     */
    static clear(): void;
}
