(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('popper.js')) :
  typeof define === 'function' && define.amd ? define(['popper.js'], factory) :
  (global = global || self, global.uiguide = factory(global.Popper));
}(this, (function (Popper) { 'use strict';

  Popper = Popper && Popper.hasOwnProperty('default') ? Popper['default'] : Popper;

  function updateHighlight(elements) {
      var _a;
      var _b = elements.highlight, backdrop = _b.backdrop, box = _b.box;
      var targetRect = elements.target.getBoundingClientRect();
      var backdropRect = backdrop.getBoundingClientRect();
      box.style.left = targetRect.left - backdropRect.left + 'px';
      box.style.top = targetRect.top - backdropRect.top + 'px';
      box.style.width = targetRect.width + 'px';
      box.style.height = targetRect.height + 'px';
      (_a = elements.popper) === null || _a === void 0 ? void 0 : _a.update();
  }

  var Config = /** @class */ (function () {
      function Config() {
          this.data = {
              events: {
                  onElementsReady: undefined,
                  onHighlightUpdate: updateHighlight,
                  onTargetFound: undefined,
              },
              highlightOptions: {
                  autofocus: true,
                  clickable: true,
                  highlightUpdateDelay: 0,
                  popper: true,
                  popperRef: 'highlight-target',
                  wait: true,
              },
          };
      }
      Config.prototype.update = function (config) {
          this.extend(this.data, config);
      };
      Config.prototype.extend = function (target, source) {
          for (var key in source) {
              if (target.hasOwnProperty(key)) {
                  if (typeof source[key] === 'object') {
                      this.extend(target[key], source[key]);
                  }
                  else {
                      target[key] = source[key];
                  }
              }
          }
      };
      return Config;
  }());

  var UI = /** @class */ (function () {
      function UI() {
          this.attrPrefix = 'uig';
          this.highlight = {};
      }
      UI.prototype.toggleBodyAttr = function (on) {
          if (on === void 0) { on = true; }
          this.toggleAttrs(document.body, { on: on });
      };
      UI.prototype.sanitizeTarget = function () {
          if (this.target) {
              this.toggleAttrs(this.target, {
                  clickable: false,
                  'non-positioned': false,
                  target: false,
              });
          }
      };
      UI.prototype.setTarget = function (target, options) {
          this.target = target;
          this.toggleAttrs(target, {
              clickable: options.clickable,
              'non-positioned': !options.positioned,
              target: true,
          });
          if (options.focus) {
              target.focus();
          }
      };
      UI.prototype.unsetTarget = function () {
          this.sanitizeTarget();
          delete this.target;
      };
      UI.prototype.sanitizeHighlight = function () {
          if (this.highlight.backdrop) {
              this.toggleAttrs(this.highlight.backdrop, { show: false });
          }
          if (this.highlight.box) {
              this.toggleAttrs(this.highlight.box, { show: false });
          }
      };
      UI.prototype.setHightlight = function (options) {
          if (!this.highlight.backdrop) {
              this.highlight.backdrop = document.createElement('div');
              this.toggleAttrs(this.highlight.backdrop, { 'highlight-backdrop': true });
          }
          if (!this.highlight.box) {
              this.highlight.box = document.createElement('div');
              this.toggleAttrs(this.highlight.box, { 'highlight-box': true });
              this.highlight.backdrop.appendChild(this.highlight.box);
          }
          options.parent.appendChild(this.highlight.backdrop);
      };
      UI.prototype.unsetHighlight = function () {
          var _a, _b, _c, _d;
          (_b = (_a = this.highlight.backdrop) === null || _a === void 0 ? void 0 : _a.parentNode) === null || _b === void 0 ? void 0 : _b.removeChild(this.highlight.backdrop);
          delete this.highlight.backdrop;
          (_d = (_c = this.highlight.box) === null || _c === void 0 ? void 0 : _c.parentNode) === null || _d === void 0 ? void 0 : _d.removeChild(this.highlight.box);
          delete this.highlight.box;
      };
      UI.prototype.toggleHightlight = function (show) {
          if (show === void 0) { show = true; }
          if (this.highlight.backdrop) {
              this.toggleAttrs(this.highlight.backdrop, { show: show });
          }
          if (this.highlight.box) {
              this.toggleAttrs(this.highlight.box, { show: show });
          }
      };
      UI.prototype.setPopup = function (options) {
          var _a;
          this.popup = document.createElement('div');
          this.toggleAttrs(this.popup, { popup: true });
          var popperRef = options.popperRef === 'highlight-box' ? this.highlight.box : this.target;
          if (!popperRef) {
              throw new Error( /* istanbul ignore next */
                      "Popper reference is set to '" + options.popperRef + "' but highlight's " + (options.popperRef === 'highlight-box' ? 'box' : 'target') + " element is not available."
                  );
          }
          document.body.appendChild(this.popup);
          (_a = this.popper) === null || _a === void 0 ? void 0 : _a.destroy();
          this.popper = new Popper(popperRef, this.popup, options.popperOptions);
      };
      UI.prototype.unsetPopup = function () {
          var _a, _b, _c;
          (_a = this.popper) === null || _a === void 0 ? void 0 : _a.destroy();
          delete this.popper;
          (_c = (_b = this.popup) === null || _b === void 0 ? void 0 : _b.parentNode) === null || _c === void 0 ? void 0 : _c.removeChild(this.popup);
          delete this.popup;
      };
      UI.prototype.togglePopup = function (show) {
          if (show === void 0) { show = true; }
          if (this.popup) {
              this.toggleAttrs(this.popup, { show: show });
          }
      };
      UI.prototype.getUpdateSchedulerRequiredElements = function () {
          if (!this.target) {
              throw new Error( /* istanbul ignore next */
                      'Unable to get the required elements of UpdateScheduler.\n' +
                          'Target element currently not available.'
                  );
          }
          if (!this.highlight.backdrop || !this.highlight.box) {
              throw new Error( /* istanbul ignore next */
                      'Unable to get the required elements of UpdateScheduler.\n' +
                          'Highlight elements currently not available.'
                  );
          }
          return {
              highlight: { backdrop: this.highlight.backdrop, box: this.highlight.box },
              popper: this.popper,
              popup: this.popup,
              target: this.target,
          };
      };
      /**
       * Toggles multiple attributes on element.
       *
       * **Usage:**
       * ```typescript
       * toggleAttrs(element, {
       *   'data-show': true, // Sets a data-show attribute to element
       *   'data-hide': false, // Removes data-hide attribute to element
       * })
       * ```
       */
      UI.prototype.toggleAttrs = function (element, attrsObject) {
          for (var attr in attrsObject) {
              if (attrsObject.hasOwnProperty(attr)) {
                  var attrName = this.attrPrefix + '-' + attr;
                  if (attrsObject[attr]) {
                      element.setAttribute(attrName, '');
                  }
                  else {
                      element.removeAttribute(attrName);
                  }
              }
          }
      };
      return UI;
  }());

  var UIUpdateScheduler = /** @class */ (function () {
      function UIUpdateScheduler(ui) {
          this.ui = ui;
          this.scheduledUpdate = {
              animationFrameId: 0,
              timeoutId: 0,
          };
      }
      UIUpdateScheduler.prototype.schedUpdate = function (updateFn, delay) {
          var elements = this.ui.getUpdateSchedulerRequiredElements();
          this.cancelCurrentScheduledUpdate();
          this.update(elements, updateFn, delay);
      };
      UIUpdateScheduler.prototype.cancelCurrentScheduledUpdate = function () {
          window.cancelAnimationFrame(this.scheduledUpdate.animationFrameId);
          window.clearTimeout(this.scheduledUpdate.timeoutId);
      };
      UIUpdateScheduler.prototype.update = function (elements, updateFn, delay) {
          var _this = this;
          this.scheduledUpdate.animationFrameId = window.requestAnimationFrame(function () {
              updateFn(elements);
              _this.scheduledUpdate.timeoutId = window.setTimeout(function () {
                  _this.update(elements, updateFn, delay);
              }, delay);
          });
      };
      return UIUpdateScheduler;
  }());

  function isElementPositioned(element) {
      // 1st check.
      // Using the element's computed style position.
      if (window.getComputedStyle(element).position !== 'static') {
          return true;
      }
      // 2nd check.
      // Using the element's first child.
      if (element.firstElementChild instanceof HTMLElement) {
          return element.firstElementChild.offsetParent === element;
      }
      // 3rd check.
      // Mimicking the 2nd check by appending a temporary element.
      var tempElement = document.createElement('span');
      tempElement.style.visibility = 'hidden';
      element.append(tempElement);
      var isPositioned = tempElement.offsetParent === element;
      tempElement.remove();
      return isPositioned;
  }

  function defer() {
      var deferred = {};
      deferred.promise = new Promise(function (resolve, reject) {
          deferred.resolve = resolve;
          deferred.reject = reject;
      });
      return deferred;
  }

  function queryWaitElement(config, options) {
      var _a, _b, _c;
      var deferred = defer();
      if (options.target instanceof Element) {
          deferred.resolve(options.target);
          return deferred;
      }
      var context = options.context || document;
      var element = context.querySelector(options.target);
      if (element) {
          deferred.resolve(element);
          return deferred;
      }
      var waitOptions = (_a = options.wait, (_a !== null && _a !== void 0 ? _a : config.highlightOptions.wait));
      if (!waitOptions) {
          deferred.reject("Can't find the target element.");
          return deferred;
      }
      var delay = typeof waitOptions === 'boolean' ? 0 : (_b = waitOptions.delay, (_b !== null && _b !== void 0 ? _b : 0));
      var max = typeof waitOptions === 'boolean' ? Infinity : (_c = waitOptions.max, (_c !== null && _c !== void 0 ? _c : Infinity));
      var selector = options.target;
      var elapsedTime = 0;
      var interval = window.setInterval(function () {
          elapsedTime += delay;
          if (elapsedTime >= max) {
              deferred.reject("Max wait reached. Can't find the element.");
          }
          element = context.querySelector(selector);
          if (element) {
              deferred.resolve(element);
          }
      }, delay);
      // tslint:disable-next-line: no-floating-promises
      deferred.promise.finally(function () {
          window.clearInterval(interval);
      });
      return deferred;
  }

  // Services
  var config = new Config();
  var ui = new UI();
  var updateScheduler = new UIUpdateScheduler(ui);
  var operation = null;
  var UIGuide = /** @class */ (function () {
      function UIGuide() {
      }
      /**
       * Configure the global highlight settings.
       *
       * **Important**: *Updating the configuration while there's an on going*
       * *highlight operation is prohibited.*
       *
       * @param config New configuration.
       */
      UIGuide.configure = function (configuration) {
          if (operation) {
              throw new Error( /* istanbul ignore next */
                      'Updating of configuration is forbidden while there is a pending highlight operation.'
                  );
          }
          config.update(configuration);
      };
      /**
       * Highlight an element from the page.
       *
       * **Tip:** *It is possible to highlight a new element without*
       * *`unhighlight`ing the current highlighted. Just make sure to wait your*
       * *first highlight operation before you can highlight a new one, otherwise*
       * *it will throw an error.*
       *
       * *Example:*
       * ```javascript
       * UIGuide.highlight('#target-1').then(() => {
       *   UIGuide.highlight('#target-2')
       * })
       * ```
       *
       * @param opts Target element or highlight options for advanced highlighting.
       */
      UIGuide.highlight = function (opts) {
          if (operation) {
              throw new Error( /* istanbul ignore next */
                      'UIGuide currently has a pending highlight operation.\n' +
                          'Make sure that the operation is finished before you highlight again.'
                  );
          }
          ui.sanitizeHighlight();
          ui.sanitizeTarget();
          var options = opts instanceof Element || typeof opts === 'string'
              ? { target: opts }
              : opts;
          var events = options.events || {};
          operation = queryWaitElement(config.data, options);
          return operation.promise
              .then(function (target) {
              var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
              ui.toggleBodyAttr();
              ui.setTarget(target, {
                  clickable: (_a = options.clickable, (_a !== null && _a !== void 0 ? _a : config.data.highlightOptions.clickable)),
                  focus: (_b = options.autofocus, (_b !== null && _b !== void 0 ? _b : config.data.highlightOptions.autofocus)),
                  positioned: isElementPositioned(target),
              });
              (_d = (_c = events).onTargetFound) === null || _d === void 0 ? void 0 : _d.call(_c, target);
              (_f = (_e = config.data.events).onTargetFound) === null || _f === void 0 ? void 0 : _f.call(_e, target);
              ui.setHightlight({
                  parent: target.offsetParent || document.body,
              });
              var popper = (_g = options.popper, (_g !== null && _g !== void 0 ? _g : config.data.highlightOptions.popper));
              ui.unsetPopup();
              if (popper) {
                  ui.setPopup({
                      popperOptions: typeof popper === 'boolean' ? undefined : popper,
                      popperRef: (_h = options.popperRef, (_h !== null && _h !== void 0 ? _h : config.data.highlightOptions.popperRef)),
                  });
              }
              var requiredElements = ui.getUpdateSchedulerRequiredElements();
              (_k = (_j = events).onElementsReady) === null || _k === void 0 ? void 0 : _k.call(_j, requiredElements);
              (_m = (_l = config.data.events).onElementsReady) === null || _m === void 0 ? void 0 : _m.call(_l, requiredElements);
              ui.toggleHightlight();
              ui.togglePopup();
              updateScheduler.schedUpdate((_o = events.onHighlightUpdate, (_o !== null && _o !== void 0 ? _o : config.data.events.onHighlightUpdate)), (_p = options.highlightUpdateDelay, (_p !== null && _p !== void 0 ? _p : config.data.highlightOptions.highlightUpdateDelay)));
              return {
                  element: target,
                  unhighlight: function () {
                      UIGuide.clear();
                  },
              };
          })
              .finally(function () {
              operation = null;
          });
      };
      /**
       * Unhighlight the current highlighted element.
       *
       * You usually invoke this function after you're done highlighting all your
       * target elements.
       *
       * *Example:*
       * ```typescript
       * UIGuide.highlight('#first-target').then(() => {
       *   UIGuide.highlight('#final-target').then((highlighted) => {
       *     UIGuide.clear() // or highlighted.unhighlight()
       *   })
       * })
       * ```
       */
      UIGuide.clear = function () {
          updateScheduler.cancelCurrentScheduledUpdate();
          ui.toggleBodyAttr(false);
          ui.unsetPopup();
          ui.unsetHighlight();
          ui.unsetTarget();
      };
      return UIGuide;
  }());

  return UIGuide;

})));
//# sourceMappingURL=ui-guide.js.map
